/*
 * rotor.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <linux/dvb/frontend.h>
#include <sys/ioctl.h>
#include <vdr/config.h>
#include <vdr/diseqc.h>
#include <vdr/plugin.h>
#include <vdr/sources.h>
#include <vdr/menuitems.h>
#include <vdr/dvbdevice.h>
#include <vdr/device.h>
#include <vdr/channels.h>
#include <vdr/status.h>
#include <vdr/interface.h>
#include <vdr/osd.h>
#if (VDRVERSNUM>=10300)
#include <vdr/pat.h>
#endif
#include <math.h>
#include "i18n.h"
#define DEV_DVB_FRONTEND "frontend"

static const char *VERSION        = "0.0.6c";
static const char *DESCRIPTION    = "Steuerung eines Rotors";
static const char *MAINMENUENTRY  = "Rotor";


int DvbKarte=0;
int AByte=49;
int wiederholen=false;
int Lat=0;
int Long=0;
int SN=false;
int EW=true;
int SLOF=11700;
int FrequLo=9750;
int FrequHi=10600;
int UseGotoX=false;
int ShowInfo=false;
struct sourced {
        char name[50];
        int Pos;
        int code;
};
sourced SourceD[100];
  int anz=0;
int PosG=0;

// --- cStatusMonitor -------------------------------------------------------

class cStatusMonitor:public cStatus {
private:
  int ActualSource;
  bool transfer;
protected:
  virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
public:
  cStatusMonitor();
};

cStatusMonitor::cStatusMonitor()

{
  ActualSource=0;
  transfer=false;
}

void cStatusMonitor::ChannelSwitch(const cDevice *Device, int ChannelNumber)
{
  if (ChannelNumber) {
     if ((Device!=cDevice::PrimaryDevice()) || (cDevice::ActualDevice()==cDevice::PrimaryDevice()) || (cDevice::PrimaryDevice()->cDevice::HasProgramme()) && transfer) {
     cChannel channel = *Channels.GetByNumber(ChannelNumber);
     if (DvbKarte==Device->CardIndex())
     if (channel.Source()!=ActualSource) {
      if (UseGotoX) {
        ActualSource=channel.Source();
     static char buffer[PATH_MAX];
     snprintf(buffer, sizeof(buffer), "%s%d/%s%d", "/dev/dvb/adapter",DvbKarte,"frontend",0);
     int fd_frontend = open(buffer,0);
     int gotoXTable[10] = { 0x00, 0x02, 0x03, 0x05, 0x06, 0x08, 0x0A, 0x0B, 0x0D, 0x0E };
     int satlong = channel.Source() & ~0xC800;
     if (channel.Source() & 0x0800)
        satlong = satlong * (-1);
     int signEW=1;
     if (EW)
          signEW=-1;
     int signSN=1;
     if (SN)
         signSN=-1;
     double azimuth=M_PI+atan(tan((satlong-Long*signEW)*M_PI/1800)/sin(Lat*signSN*M_PI/1800));
     double x=acos(cos((satlong-Long*signEW)*M_PI/1800)*cos(Lat*signSN*M_PI/1800));
     double elevation=atan((cos(x)-0.1513)/sin(x));
     double SatHourangle=180+atan((-cos(elevation)*sin(azimuth))/(sin(elevation)*cos(Lat*M_PI/1800)-cos(elevation)*sin(Lat*M_PI/1800)*cos(azimuth)))*180/M_PI;
     struct dvb_diseqc_master_cmd cmd;
     cmd.msg[0]=0xe0;
     cmd.msg[1]=0x31;
     cmd.msg[2]=0x6e;
     int tmp=(int)(fabs(180-SatHourangle)*10);
     tmp=(tmp/10)*0x10 + gotoXTable[ tmp % 10 ];
     cmd.msg[4]=(tmp%0x0100);
     cmd.msg[3]=(tmp/0x0100);
     if (SatHourangle < 180)
         cmd.msg[3] |= 0xe0;
        else
         cmd.msg[3] |= 0xd0;
     cmd.msg_len=5;
     CHECK(ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd));
     cmd.msg[0]=0xe1;
     if (wiederholen)
     for (int i=1 ; i<=2 ; i++) {
         usleep(15000);
         CHECK(ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd));
        }
      }

     if (ShowInfo && (cDevice::PrimaryDevice()==cDevice::ActualDevice())) {

#if (VDRVERSNUM>=10307)
  cOsd *osd=NULL;
    osd = cOsdProvider::NewOsd(Setup.OSDLeft, 480);
  if (osd) {
     tArea Area = { 0, 0, 655,25,  2 };
     osd->SetAreas(&Area, 1);
     osd->DrawRectangle(0, 0,655,25,clrGreen);
     osd->DrawText((Setup.OSDWidth-50*sizeof(tr("Rotating Dish")))/2,0,tr("Rotating Dish"),clrBlack,clrGreen,cFont::GetFont(fontOsd));
     osd->Flush();
     }

#else

     Interface->Open(Setup.OSDwidth,-1);
     Interface->Status(tr("Rotating Dish"),clrBlack,clrGreen);
     Interface->Flush();
#endif
     eKeys Key = kNone;
     static char buffer[PATH_MAX];
     snprintf(buffer, sizeof(buffer), "%s%d/%s%d", "/dev/dvb/adapter",DvbKarte,"frontend",0);
     int fd_frontend = open(buffer,0);
     for (;;) {
           fe_status_t status = fe_status_t(0);
               static char buffer[PATH_MAX];
     snprintf(buffer, sizeof(buffer), "%s%d/%s%d", "/dev/dvb/adapter",DvbKarte,"frontend",0);

          usleep(15);
          CHECK(ioctl(fd_frontend, FE_READ_STATUS, &status));
          Key = cRemote::Get(10);
          cRemote::Put(Key);
          if ((status & FE_HAS_SYNC) | (Key != kNone))
             break;

        }
#if (VDRVERSNUM>=10307)
  delete osd;
#else
     Interface->Status(NULL);
     Interface->Close();
#endif
     }
      }
     }
       if ((cDevice::ActualDevice()!=cDevice::PrimaryDevice()) && (Device==cDevice::PrimaryDevice()) )
        transfer=true;
       if ((cDevice::ActualDevice()==cDevice::PrimaryDevice()) && (Device==cDevice::PrimaryDevice())) transfer=false;
   }
}



// --- cMenuEditIntpItem ----------------------------------------------------

class cMenuEditIntpItem : public cMenuEditIntItem {
protected:
  virtual void Set(void);
  const char *falseString, *trueString;
  int *value2;
public:
  cMenuEditIntpItem(const char *Name, int *Value, int Min = 0, int Max = INT_MAX, int *Value2=0, const char *FalseString = "", const char *TrueSting = NULL);
  virtual eOSState ProcessKey(eKeys Key);
};

void cMenuEditIntpItem::Set(void)
{
  char buf[16];
  snprintf(buf, sizeof(buf), "%d.%d %s", *value/10, *value % 10, *value2 ? trueString : falseString);
  SetValue(buf);
}

cMenuEditIntpItem::cMenuEditIntpItem(const char *Name, int *Value, int Min, int Max,int *Value2, const char *FalseString,const char *TrueString):cMenuEditIntItem(Name, Value, Min, Max)
{
  value = Value;
  value2= Value2;
  trueString = TrueString;
  falseString = FalseString;
  min = Min;
  max = Max;
  Set();
}

eOSState cMenuEditIntpItem::ProcessKey(eKeys Key)
{
  eOSState state = cMenuEditItem::ProcessKey(Key);

  if (state == osUnknown) {
     int newValue = *value;
     int newValue2= *value2;
     Key = NORMALKEY(Key);
     switch (Key) {
       case kNone: break;
       case k0 ... k9:
            if (fresh) {
               *value = 0;
               fresh = false;
               }
            newValue = *value * 10 + (Key - k0);
            break;
       case kLeft: // TODO might want to increase the delta if repeated quickly?
            newValue2 = 0;
            fresh = true;
            break;
       case kRight:
            newValue2 = 1;
            fresh = true;
            break;
       default:
            if (*value < min) { *value = min; Set(); }
            if (*value > max) { *value = max; Set(); }
            return state;
       }
     if ((!fresh || min <= newValue) && newValue <= max) {
        *value = newValue;
        *value2 = newValue2;
        Set();
        }
     state = osContinue;
     }
  return state;
}


// --- cMenuSetupRotor ------------------------------------------------------

class cMenuSetupRotor : public cMenuSetupPage {
private:
  int newDvbKarte;
  int newAByte;
  int newwiederholen;
  int newLat;
  int newLong;
  int newSN;
  int newEW;
  int newSLOF;
  int newFrequLo;
  int newFrequHi;
  int newUseGotoX;
  int newShowInfo;
protected:
  virtual void Store(void);
public:
  cMenuSetupRotor(void);
  };

cMenuSetupRotor::cMenuSetupRotor(void)
{
  newDvbKarte=DvbKarte+1;
  newAByte=AByte-18;
  newwiederholen=wiederholen;
  newLat=Lat;
  newLong=Long;
  newSN=SN;
  newEW=EW;
  newSLOF=SLOF;
  newFrequLo=FrequLo;
  newFrequHi=FrequHi;
  newUseGotoX=UseGotoX;
  newShowInfo=ShowInfo;
  Add(new cMenuEditIntItem( tr("Card, connected with motor"), &newDvbKarte,1,cDevice::NumDevices()));
  Add(new cMenuEditBoolItem(tr("Repeat DiSEqC-Commands?"),   &newwiederholen));
  Add(new cMenuEditIntItem( tr("Address-Byte"),              &newAByte,30,31));
  Add(new cMenuEditIntpItem( tr("Latitude"),                 &newLat,0,900,&newSN,tr("North"),tr("South")));
  Add(new cMenuEditIntpItem( tr("Longitude"),                &newLong,0,1800,&newEW,tr("West"),tr("East")));
  Add(new cMenuEditIntItem( tr("Setup.LNB$SLOF (MHz)"),               &newSLOF));
  Add(new cMenuEditIntItem( tr("Setup.LNB$Low LNB frequency (MHz)"),  &newFrequLo));
  Add(new cMenuEditIntItem( tr("Setup.LNB$High LNB frequency (MHz)"), &newFrequHi));
  Add(new cMenuEditBoolItem(tr("Use GotoX on channel switch"),&newUseGotoX));
  Add(new cMenuEditBoolItem(tr("Setup.OSD$Info on channel switch"),&newShowInfo));

}

void cMenuSetupRotor::Store(void)
{
  SetupStore("DVB-Karte", DvbKarte=newDvbKarte-1);
  SetupStore("AdressByte", AByte=newAByte+18);
  SetupStore("Wiederholen", wiederholen=newwiederholen);
  SetupStore("Latitude", Lat=newLat);
  SetupStore("SouthNorth", SN=newSN);
  SetupStore("Longitude", Long=newLong);
  SetupStore("EastWest", EW=newEW);
  SetupStore("SLOF",SLOF=newSLOF);
  SetupStore("FrequLo",FrequLo=newFrequLo);
  SetupStore("FrequHi",FrequHi=newFrequHi);
  SetupStore("UseGotoX",UseGotoX=newUseGotoX);
  SetupStore("ShowInfo",ShowInfo=newShowInfo);
}

// -- cMainMenuRotor --------------------------------------------------------

class cMainMenuRotor : public cOsdObject {
private:
  int ECon,WCon,SCon,GCon,RCon,FCon,SyCon,paint,meni,zmeni,conf,repeat,confa,oldupdate,HasSwitched,fd_frontend,StepsE,StepsW,PosG,PosS,PosR,gbyt1,gbyt2,SNr,Frequenz,Symbolrate;
  char *Pol;
  cChannel *OldChannel;
  cChannel Channel;
#if (VDRVERSNUM>=10307) 
  cOsd *osd;
  tColor color;
#else 
  cOsdBase *osd;
  eDvbColor color;
#endif 
  int Poszuw(int Position);
  int Sourcezuw(int code);
  void DiseqcCommand(int KNr);

public:
  cMainMenuRotor(void);
  ~cMainMenuRotor();
  virtual void Show(void);
  virtual eOSState ProcessKey(eKeys Key);
  void DisplaySignalInfoOnOsd(fe_status_t status,unsigned int ber,unsigned int ss,float ssdBm,unsigned int snr,float snrdB,unsigned int unc);
  void GetSignalInfo(         fe_status_t &status,
                             unsigned int &BERChip,
                             unsigned int &SSChip,
                             unsigned int &SNRChip,
                             unsigned int &UNCChip  );
  bool Signal(int Frequenz, char *Pol, int Symbolrate);
};


int cMainMenuRotor::Poszuw(int Position)

{
  for (int i=1; i<=anz ; i++ ) {
    if (SourceD[i].Pos==Position && Position!=0) {
        return i;
        break;
        }
  }
  return 0;
}

int cMainMenuRotor::Sourcezuw(int code)

{
  for (int i=1; i<=anz ; i++) {
    if (SourceD[i].code==code) {
        return i;
        break;
        }
      }
  return 0;
}

cMainMenuRotor::cMainMenuRotor(void)
{
#if (VDRVERSNUM>=10300 )
  oldupdate=Setup.UpdateChannels;
  Setup.UpdateChannels=0;
#endif
  OldChannel=Channels.GetByNumber(cDevice::GetDevice(DvbKarte)->CurrentChannel());
  repeat=HasSwitched=false;
  conf=meni=zmeni=StepsE=StepsW=SNr=1;
  paint=gbyt1=gbyt2=0;
  osd=NULL;
  static char buffer[PATH_MAX];
  snprintf(buffer, sizeof(buffer), "%s%d/%s%d", "/dev/dvb/adapter",DvbKarte,"frontend",0);
  fd_frontend = open(buffer,0);
  cChannel *Channel=Channels.GetByNumber(cDevice::GetDevice(DvbKarte)->CurrentChannel(),0);
  SNr=Sourcezuw(Channel->Source());
  PosG=SourceD[SNr].Pos;
  PosS=SourceD[SNr].Pos;
  PosR=SourceD[SNr].Pos;
  Frequenz=(*Channel).Frequency();
  if ((*Channel).Polarization() == 'v' || (*Channel).Polarization() == 'V')
      Pol="V";
   else Pol="H";
  Symbolrate=(*Channel).Srate();
}

cMainMenuRotor::~cMainMenuRotor()
{
  delete osd;
  if (HasSwitched) {
     cDevice::GetDevice(DvbKarte)->SwitchChannel(OldChannel,true);
     }
#if (VDRVERSNUM>=10300 )
  Setup.UpdateChannels=oldupdate;
#endif
}


void cMainMenuRotor::Show(void)
{
        fe_status_t status;
        unsigned int BERChip=0;
        unsigned int SSChip=0;
        unsigned int SNRChip=0;
        unsigned int UNCChip=0;
        GetSignalInfo(status,BERChip,SSChip,SNRChip,UNCChip);
        float snrdB = 0.0;
        float ssdBm = 0.0;
        unsigned int unc = UNCChip;
        unsigned int ber = BERChip;
        unsigned int ss = SSChip/655;
        unsigned int snr = SNRChip/655;
        ssdBm = (logf(((double)SSChip)/65535)*10.8);
        if (SNRChip>57000)
         snrdB = (logf((double)SNRChip/6553.5)*10.0);
        else
         snrdB = (-3/(logf(SNRChip/65535.0)));
        DisplaySignalInfoOnOsd(status,ber,ss,ssdBm,snr,snrdB,unc);

}


void cMainMenuRotor::DiseqcCommand(int KNr)
{
  struct diseqc_cmd {
        struct dvb_diseqc_master_cmd cmd;
        uint32_t wait;
  };
  struct dvb_diseqc_master_cmd switch_cmds[] = {
        { { 0xe0,AByte, 0x60, 0x00, 0x00, 0x00 }, 3 },  //0 Stop Positioner movement
        { { 0xe0,AByte, 0x63, 0x00, 0x00, 0x00 }, 3 },  //1 Disable Limits
        { { 0xe0,AByte, 0x66, 0x00, 0x00, 0x00 }, 3 },  //2 Set East Limit
        { { 0xe0,AByte, 0x67, 0x00, 0x00, 0x00 }, 3 },  //3 Set West Limit
        { { 0xe0,AByte, 0x68, 0x00, 0x00, 0x00 }, 4 },  //4 Drive Motor East continously
        { { 0xe0,AByte,0x68,256-StepsE,0x00,0x00},4 },  //5 Drive Motor East nn steps
        { { 0xe0,AByte,0x69,256-StepsW,0x00,0x00},4 },  //6 Drive Motor West nn steps
        { { 0xe0,AByte, 0x69, 0x00, 0x00, 0x00 }, 4 },  //7 Drive Motor West continously
        { { 0xe0,AByte, 0x6a, PosS, 0x00, 0x00 }, 4 },  //8 Store nn
        { { 0xe0,AByte, 0x6b, PosG, 0x00, 0x00 }, 4 },   //9 Goto nn
        { { 0xe0,AByte, 0x6f, PosR, 0x00 ,0x00 }, 4 },  //10 Recalculate Position
        { { 0xe0,AByte, 0x6a, 0x00, 0x00, 0x00 }, 4 },  //11 Enable Limits
        { { 0xe0,AByte, 0x6e,gbyt1,gbyt2, 0x00 }, 5 }   //12 Gotoxx
};
  struct dvb_diseqc_master_cmd cmd;
  cmd = switch_cmds[KNr];
  CHECK(ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd));
  if (wiederholen) {
    cmd.msg[0]=0xe1;
      CHECK(ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd));
      usleep(15000);
      CHECK(ioctl(fd_frontend, FE_DISEQC_SEND_MASTER_CMD, &cmd));
     }
  Signal(Frequenz,Pol,Symbolrate);
}

eOSState cMainMenuRotor::ProcessKey(eKeys Key)
{
  int cmdnr=0;
  eOSState state = cOsdObject::ProcessKey(Key);
  if (state == osUnknown) {
  int om,omz;
  om=meni;
  omz=zmeni;
  switch(Key) {
    case kLeft: conf=1;
                if ((meni!=0) && (zmeni<5))
                meni--;
                if ((zmeni==5) && (SNr!=1)) {
                   SNr--;
                   PosG=PosS=PosR=SourceD[SNr].Pos;
                   }
                if (zmeni==6)
                   Pol="V";
                break;
    case kRight:conf=1;
                if ((meni!=2) && (zmeni<5))
                meni++;
                if ((zmeni==5) && (SNr!=anz)) {
                   SNr++;
                   PosG=PosS=PosR=SourceD[SNr].Pos;
                   }
                if (zmeni==6)
                   Pol="H";
                break;
    case kUp:conf=1;
             if (zmeni!=1)
                zmeni--;
                break;
    case kDown: conf=1;
                if (zmeni<4)
                zmeni++;
                #if (VDRVERSNUM>=10300)
                else if (zmeni>=4 && zmeni<8) {
                #else 
                else if (zmeni>=4 && zmeni<7) {
                #endif
                  zmeni++;
                  meni=1;
                  }
                break;
    case k0 ... k9:  conf=1;
                   if ((meni==0) && (zmeni==3)) {
                           if (ECon==0) {
                                   StepsE = Key - 11;
                                   ECon++;
                           }
                           else if (ECon==1) {
                                   StepsE = StepsE*10 + Key - 11;
                                   ECon++;
                           }
                   }
                   if ((meni==2) && (zmeni==3)) {
                           if (WCon==0) {
                                   StepsW = Key - 11;
                                   WCon++;
                           }
                           else if (WCon==1) {
                                   StepsW = StepsW*10 + Key - 11;
                                   WCon++;
                           }
                   }
                   if ((meni==0) && (zmeni==2)) {
                           if (RCon==0) {
                                   PosR = Key - 11;
                                   RCon++;
                           }
                           else if (RCon==1) {
                                   PosR = PosR*10 + Key - 11;
                                   RCon++;
                           }
                   }
                   if ((meni==1) && (zmeni==2)) {      
			   if (GCon==0) {
                                   PosG = Key - 11;
				   GCon++;
                           }
                           else if (GCon==1) {
                                   PosG = PosG*10 + Key - 11;
                                   GCon++;
                           }
                   }
                   if ((meni==2) && (zmeni==2)) {
                           if (SCon==0) {
                                   PosS = Key - 11;
                                   SCon++;
                           }
                           else if (SCon==1) {
                                   PosS = PosS*10 + Key - 11;
                                   SCon++;
                           }
                   }
                   if (zmeni==6) {
                           if (FCon==0) {
                                   Frequenz= Key - 11;
                                   FCon++;
                           }
                           else if (FCon<5) {
                                   Frequenz=Frequenz*10 + Key - 11;
                                   FCon++;
                           }
                   }
                   if (zmeni==7) {
                           if (SyCon==0) {
                                   Symbolrate= Key - 11;
                                   SyCon++;
                           }
                           else if (SyCon<5) {
                                   Symbolrate=Symbolrate*10 + Key - 11;
                                   SyCon++;
                           }
                   }
                   if ((zmeni==2) && (meni==1) && (PosG!=0))
                           SNr=Poszuw(PosG);
                   if ((zmeni==2) && (meni==2) && (PosS!=0))
                           SNr=Poszuw(PosS);
                   break;
    case kOk:cmdnr=50;
             #if (VDRVERSNUM>=10300 )
             Setup.UpdateChannels=0;
             #endif
             if (conf==0) {
                     conf=1;
                     cmdnr=confa;
             }
             else {
                     if (zmeni==1) {
                             if (meni==0)
                                     cmdnr = 4;
                             if (meni==1)
                                     cmdnr = 0;
                             if (meni==2)
                                     cmdnr = 7;
                     }
                     if (zmeni==2) {
                             if (meni==0) {
                                     confa = 10;
                                     conf=0;
                             }
                             if (meni==1) {
                                     cmdnr = 9;
                                     PosR = PosG;
                             }
                             if (meni==2) {
                                     confa = 8;
                                     conf=0;
                                     PosR = PosG;
                             }
                     }
                     if (zmeni==3) {
                             if (meni==0)
                                     cmdnr = 5;
                             if (meni==1)
                                     cmdnr = 1;
                             if (meni==2)
                                     cmdnr = 6;
                     }
                     if (zmeni==4) {
                             if (meni==0)
                                     cmdnr = 2;
                             if (meni==1)
                                     cmdnr = 11;
                             if (meni==2)
                                     cmdnr = 3;
                     }
                     if (zmeni==5) {
                             cmdnr = 12;
                             int satlong = SourceD[SNr].code & ~0xC800;
                             if (SourceD[SNr].code & 0x0800)
                                     satlong = satlong * (-1);
                             int signEW=1;
                             if (EW)
                                signEW=-1;
                             int signSN=1;
                             if (SN)
                             signSN=-1;
                             double azimuth=M_PI+atan(tan((satlong-Long*signEW)*M_PI/1800)/sin(Lat*signSN*M_PI/1800));
                             double x=acos(cos((satlong-Long*signEW)*M_PI/1800)*cos(Lat*signSN*M_PI/1800));
                             double elevation=atan((cos(x)-0.1513)/sin(x));
                             double SatHourangle=180+atan((-cos(elevation)*sin(azimuth))/(sin(elevation)*cos(Lat*M_PI/1800)-cos(elevation)*sin(Lat*M_PI/1800)*cos(azimuth)))*180/M_PI;
                             int gotoXTable[10] = { 0x00, 0x02, 0x03, 0x05, 0x06, 0x08, 0x0A, 0x0B, 0x0D, 0x0E };         
                             int tmp=(int)(fabs(180-SatHourangle)*10);
                             gbyt1=(tmp/10)*0x10 + gotoXTable[ tmp % 10 ];
                             gbyt2=(gbyt1%0x0100);
                             gbyt1=(gbyt1/0x0100);
                             if (SatHourangle < 180)
                                 gbyt1 |= 0xe0;
                                else
                                 gbyt1 |= 0xd0;
                }
	        #if (VDRVERSNUM>=10300)
                if (zmeni==8) {
                   Setup.UpdateChannels=4;
                   cChannel *SChannel= new cChannel; // OldChannel;
                   int tmp=OldChannel->Apid1();
                   char tmp2[1][4]={"deu"};
                   SChannel->SetPids(OldChannel->Vpid(),OldChannel->Ppid(),&tmp,tmp2,&tmp,tmp2,OldChannel->Tpid());
                   SChannel->cChannel::SetSatTransponderData(SourceD[SNr].code,Frequenz,*Pol,Symbolrate,FEC_AUTO);
                   if (cDevice::GetDevice(DvbKarte)==cDevice::ActualDevice())
                      HasSwitched=true;
                   cDevice::GetDevice(DvbKarte)->SwitchChannel(SChannel,HasSwitched);
                }
		#endif
                if ((zmeni==6) || (zmeni==7) || (zmeni==8))
		   Signal(Frequenz,Pol,Symbolrate); 
                }
                if (cmdnr!=50)
                 DiseqcCommand(cmdnr);
                ECon=0;
                WCon=0;
                RCon=0;
                GCon=0;
                SCon=0;
                FCon=0;
                SyCon=0;
                repeat=false;
                break;
    case kOk|k_Release:
                        if ((repeat) && (zmeni==1) && ((meni==0) || (meni==2)))
                          DiseqcCommand(0);
                        repeat=false;
                        break;
    case kOk|k_Repeat: repeat=true;
                        break;
    case kBack: if (conf==0)
                  conf=1;
                else
                return osEnd;
        default:
                Show();
                return state;
    }
  if ((om!=meni) || (zmeni!=omz)) {
    if ((meni==0) && (zmeni==2)) {
        RCon=0;
        if (PosR!=0)
        SNr=Poszuw(PosR);
        }
    if ((meni==1) && (zmeni==2)) {
        GCon=0;
        if (PosG!=0)
        SNr=Poszuw(PosG);
        }
    if ((meni==2) && (zmeni==2)) {
        SCon=0;
        if (PosS!=0)
        SNr=Poszuw(PosS);
        }
    if ((meni==0) && (zmeni==3))
        ECon=0;
    if ((meni==2) && (zmeni==3))
        WCon=0;
    if (zmeni==6)
        FCon=0;
    if (zmeni==7)
        SyCon=0;
   }
  }
  Show();
  return state;
}

void cMainMenuRotor::DisplaySignalInfoOnOsd(fe_status_t status,unsigned int ber,unsigned int ss,float ssdBm,unsigned int snr,float snrdB,unsigned int unc)
{
      int RedLimit=415*30/100;
      int YellowLimit=max(RedLimit,415*60/100);
      int ShowSNR = 415*snr/100;
      int ShowSS =  415*ss/100;
      int PositionY = 0;

if (paint==0) {
        paint=1;
#if (VDRVERSNUM>=10307)
      osd = cOsdProvider::NewOsd(40,50);
         tArea Area[] =  {{0,0, 655, 80, 4},
			   { 0, 81,655,400, 2},
			   { 0, 401,655,470,2}};
	 osd->SetAreas(Area, sizeof(Area) / sizeof(tArea));			
;
#else
       osd = cOsd::OpenRaw(40,50);
            osd->Create(0, 0, 655,80, 4);
            osd->Create(0,81,655,320,2);
       osd->AddColor(clrBackground);

          osd->AddColor(clrBlack);
             osd->AddColor(clrCyan);
            osd->AddColor(clrWhite);
            osd->Create(0,401,655,70,2);
            osd->AddColor(clrBackground);
            osd->AddColor(clrRed);
;
#endif
	 osd->Flush();


}
      if(osd)
      {
#if (VDRVERSNUM>=10307) 
         osd->DrawRectangle(0,0,655,350,clrGray50);
         char buf[1024];
            sprintf(buf,"SNR:");
            osd->DrawText(10, 3,buf,clrWhite,clrGray50, cFont::GetFont(fontOsd));
            osd->DrawRectangle(68, 10, 68+min(RedLimit,ShowSNR), 25, clrRed);
            if(ShowSNR>RedLimit) osd->DrawRectangle(68+RedLimit, 10, 68+min(YellowLimit,ShowSNR), 25, clrYellow);
            if(ShowSNR>YellowLimit) osd->DrawRectangle(68+YellowLimit, 10, 68+ShowSNR, 25, clrGreen);
            sprintf(buf,"%d%% = %.1fdB",snr,snrdB);
            osd->DrawText(485, 3,buf,clrWhite,clrGray50, cFont::GetFont(fontOsd));
            PositionY = PositionY + 35;
            sprintf(buf,"SS:");
            osd->DrawText(10, 3+PositionY,buf,clrWhite,clrGray50, cFont::GetFont(fontOsd));
            osd->DrawRectangle(68, 10+PositionY, 68+min(RedLimit,ShowSS), 25+PositionY, clrRed);
            if(ShowSS>RedLimit) osd->DrawRectangle(68+RedLimit, 10+PositionY, 68+min(YellowLimit,ShowSS), 25+PositionY, clrYellow);
            if(ShowSS>YellowLimit) osd->DrawRectangle(68+YellowLimit, 10+PositionY, 68+ShowSS, 25+PositionY, clrGreen);
            sprintf(buf,"%d%% = %.1fdBm",ss,ssdBm);
            osd->DrawText(485, 3+PositionY,buf,clrWhite,clrGray50, cFont::GetFont(fontOsd));
            PositionY = PositionY + 35;
         tColor VG,HG;
         VG=clrRed;
         if (status & FE_HAS_SIGNAL)
            VG=clrGreen;
         osd->DrawRectangle(100,70,110,80,VG);
         VG=clrRed;
         if (status & FE_HAS_CARRIER)
            VG=clrGreen;
         osd->DrawRectangle(150,70,160,80,VG);
         VG=clrRed;
         if (status & FE_HAS_VITERBI)
            VG=clrGreen;
         osd->DrawRectangle(200,70,210,80,VG);
         VG=clrRed;
         if (status & FE_HAS_SYNC)
            VG=clrGreen;
         osd->DrawRectangle(250,70,260,80,VG);
         int x;
         int y;
         for ( x=0 ; (x<=2) ; x++ ) {
           for ( y=1 ; (y<=4) ; y++ ) {
         if ((meni==x) && (zmeni==y)) {
            HG=clrCyan;
            VG=clrBlack;
             }
            else {
            HG=clrBlack;
            VG=clrWhite;
           }
         if (y==1) {
           if (x==0)
             sprintf(buf,tr("Drive East"));
           if (x==1)
             sprintf(buf,tr("Halt"));
           if (x==2)
             sprintf(buf,tr("Drive West"));
         }
         if (y==2) {
           if (x==0)
             snprintf(buf, sizeof(buf),"%s (%d)",tr("Recalc"),PosR);
           if (x==1)
             snprintf(buf, sizeof(buf),"%s %d",tr("Goto"),PosG);
           if (x==2)
             snprintf(buf, sizeof(buf),"%s %d",tr("Store"),PosS);
         }
         if (y==3) {
           if (x==0)
                snprintf(buf, sizeof(buf), "%d %s", StepsE,tr("Steps East"));
           if (x==1)
                sprintf(buf,tr("Disable Limits"));
           if (x==2)
                snprintf(buf, sizeof(buf), "%d %s", StepsW,tr("Steps West"));
         }
         if (y==4) {
           if (x==0)
                sprintf(buf, tr("Set East Limit"));
           if (x==1)
                sprintf(buf, tr("Enable Limits"));
           if (x==2)
                sprintf(buf, tr("Set West Limit"));
         }
         osd->DrawRectangle(15+x*210,100+30*y,215+x*210,126+30*y,HG);
         osd->DrawText(16+x*210,100+30*y,buf,VG,HG,cFont::GetFont(fontOsd));
         }
         }
        if (zmeni==5) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,SourceD[SNr].name);
        osd->DrawRectangle(15,250,635,276,HG);
        osd->DrawText(20,250,buf,VG,HG,cFont::GetFont(fontOsd));
        if (zmeni==6) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,tr("Frequency:"));
        osd->DrawRectangle(15,280,635,306,HG);
        osd->DrawText(20,280,buf,VG,HG,cFont::GetFont(fontOsd));
        snprintf(buf, sizeof(buf), "%d %s",Frequenz,Pol);
        osd->DrawText(590-(int)log10(Frequenz)*13,280,buf,VG,HG,cFont::GetFont(fontOsd));
        if (zmeni==7) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,tr("Symbolrate:"));
        osd->DrawRectangle(15,310,635,336,HG);
        osd->DrawText(20,310,buf,VG,HG,cFont::GetFont(fontOsd));
        snprintf(buf, sizeof(buf), "%d", Symbolrate);
        osd->DrawText(610-(int)log10(Symbolrate)*13,310,buf,VG,HG,cFont::GetFont(fontOsd));
  #if VDRVERSNUM>=10300
        if (zmeni==8) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,tr("Scan Transponder"));
        osd->DrawRectangle(15,340,635,366,HG);
        osd->DrawText(230,340,buf,VG,HG,cFont::GetFont(fontOsd));
   #endif
        if (conf==0) {
          osd->DrawRectangle(0,444,655,470,clrRed);
          sprintf(buf, tr(" Are you sure?"));
          osd->DrawText(240,444,buf,clrWhite,clrRed,cFont::GetFont(fontOsd));
        }
        else {
        osd->DrawRectangle(4,444,655,470,clrGray50); } 
        osd->Flush();
#else 
         osd->Fill(4,0,644,80,clrBackground);
         char buf[1024];
            sprintf(buf,"SNR:");
            osd->Text(10, 3,buf,clrWhite,clrBackground);
            osd->Fill(68, 10, 68+min(RedLimit,ShowSNR), 25, clrRed);
            if(ShowSNR>RedLimit) osd->Fill(68+RedLimit, 10, 68+min(YellowLimit,ShowSNR), 25, clrYellow);
            if(ShowSNR>YellowLimit) osd->Fill(68+YellowLimit, 10, 68+ShowSNR, 25, clrGreen);
            sprintf(buf,"%d%% = %.1fdB",snr,snrdB);
            osd->Text(485, 3,buf,clrWhite,clrBackground);
            PositionY = PositionY + 35;
            sprintf(buf,"SS:");
            osd->Text(10, 3+PositionY,buf,clrWhite,clrBackground);
            osd->Fill(68, 10+PositionY, 68+min(RedLimit,ShowSS), 25+PositionY, clrRed);
            if(ShowSS>RedLimit) osd->Fill(68+RedLimit, 10+PositionY, 68+min(YellowLimit,ShowSS), 25+PositionY, clrYellow);
            if(ShowSS>YellowLimit) osd->Fill(68+YellowLimit, 10+PositionY, 68+ShowSS, 25+PositionY, clrGreen);
            sprintf(buf,"%d%% = %.1fdBm",ss,ssdBm);
            osd->Text(485, 3+PositionY,buf,clrWhite,clrBackground);
            PositionY = PositionY + 35;
         eDvbColor VG,HG;
         VG=clrRed;
         if (status & FE_HAS_SIGNAL)
            VG=clrGreen;
         osd->Fill(100,70,110,80,VG);
         VG=clrRed;
         if (status & FE_HAS_CARRIER)
            VG=clrGreen;
         osd->Fill(150,70,160,80,VG);
         VG=clrRed;
         if (status & FE_HAS_VITERBI)
            VG=clrGreen;
         osd->Fill(200,70,210,80,VG);
         VG=clrRed;
         if (status & FE_HAS_SYNC)
            VG=clrGreen;
         osd->Fill(250,70,260,80,VG);
         int x;
         int y;
         for ( x=0 ; (x<=2) ; x++ ) {
           for ( y=1 ; (y<=4) ; y++ ) {
         if ((meni==x) && (zmeni==y)) {
            HG=clrCyan;
            VG=clrBlack;
             }
            else {
            HG=clrBlack;
            VG=clrWhite;
           }
         if (y==1) {
           if (x==0)
             sprintf(buf,tr("Drive East"));
           if (x==1)
             sprintf(buf,tr("Halt"));
           if (x==2)
             sprintf(buf,tr("Drive West"));
         }
         if (y==2) {
           if (x==0)
             snprintf(buf, sizeof(buf),"%s (%d)",tr("Recalc"),PosR);
           if (x==1)
             snprintf(buf, sizeof(buf),"%s %d",tr("Goto"),PosG);
           if (x==2)
             snprintf(buf, sizeof(buf),"%s %d",tr("Store"),PosS);
         }
         if (y==3) {
           if (x==0)
                snprintf(buf, sizeof(buf), "%d %s", StepsE,tr("Steps East"));
           if (x==1)
                sprintf(buf,tr("Disable Limits"));
           if (x==2)
                snprintf(buf, sizeof(buf), "%d %s", StepsW,tr("Steps West"));
         }
         if (y==4) {
           if (x==0)
                sprintf(buf, tr("Set East Limit"));
           if (x==1)
                sprintf(buf, tr("Enable Limits"));
           if (x==2)
                sprintf(buf, tr("Set West Limit"));
         }
         osd->Fill(15+x*210,100+30*y,215+x*210,126+30*y,HG);
         osd->Text(16+x*210,100+30*y,buf,VG,HG);
         }
         }
        if (zmeni==5) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,SourceD[SNr].name);
        osd->Fill(15,250,635,276,HG);
        osd->Text(20,250,buf,VG,HG);
        if (zmeni==6) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,tr("Frequency:"));
        osd->Fill(15,280,635,306,HG);
        osd->Text(20,280,buf,VG,HG);
        snprintf(buf, sizeof(buf), "%d %s",Frequenz,Pol);
        osd->Text(590-(int)log10(Frequenz)*13,280,buf,VG,HG);
        if (zmeni==7) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,tr("Symbolrate:"));
        osd->Fill(15,310,635,336,HG);
        osd->Text(20,310,buf,VG,HG);
        snprintf(buf, sizeof(buf), "%d", Symbolrate);
        osd->Text(610-(int)log10(Symbolrate)*13,310,buf,VG,HG);
  #if VDRVERSNUM>=10300
        if (zmeni==8) {
           VG=clrBlack;
           HG=clrCyan; }
        else {
           VG=clrWhite;
           HG=clrBlack;
                }
        sprintf(buf,tr("Scan Transponder"));
        osd->Fill(15,340,635,366,HG);
        osd->Text(230,340,buf,VG,HG);
   #endif
        if (conf==0) {
          osd->Fill(4,444,655,470,clrRed);
          sprintf(buf, tr(" Are you sure?"));
          osd->Text(240,444,buf,clrWhite,clrRed);
        }
        else {
        osd->Fill(4,444,655,470,clrBackground); }
        osd->Flush(); 
#endif 
      }
      else isyslog("Darn! Couldn't create osd");
}

void cMainMenuRotor::GetSignalInfo(fe_status_t &status,
                             unsigned int &BERChip,
                             unsigned int &SSChip,
                             unsigned int &SNRChip,
                             unsigned int &UNCChip
                                )
{
      status = fe_status_t(0);
      usleep(15);
      CHECK(ioctl(fd_frontend, FE_READ_STATUS, &status));
      usleep(15);                                                 //sleep necessary else returned values may be junk
      BERChip=0;                                    //set variables to zero before ioctl, else FE_call sometimes returns junk
      CHECK(ioctl(fd_frontend, FE_READ_BER, &BERChip));
      usleep(15);
      SSChip=0;
      CHECK(ioctl(fd_frontend, FE_READ_SIGNAL_STRENGTH, &SSChip));
      usleep(15);
      SNRChip=0;
      CHECK(ioctl(fd_frontend, FE_READ_SNR, &SNRChip));
      usleep(15);
      UNCChip=0;
      CHECK(ioctl(fd_frontend, FE_READ_UNCORRECTED_BLOCKS, &UNCChip));
}

bool cMainMenuRotor::Signal(int Frequenz, char *Pol, int Symbolrate)
{
  dvb_frontend_parameters Frontend;
  memset(&Frontend, 0, sizeof(Frontend));
  if (Frequenz < SLOF) {
     Frontend.frequency=abs(Frequenz-FrequLo) * 1000UL;
     CHECK(ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_OFF));
        }
   else {
    Frontend.frequency=abs(Frequenz-FrequHi) * 1000UL;
    CHECK(ioctl(fd_frontend, FE_SET_TONE, SEC_TONE_ON));
            }
  if (Pol=="H") {
    CHECK(ioctl(fd_frontend,FE_SET_VOLTAGE,SEC_VOLTAGE_18)); }
  else CHECK(ioctl(fd_frontend,FE_SET_VOLTAGE,SEC_VOLTAGE_13));
  Frontend.inversion=INVERSION_AUTO;
  Frontend.u.qpsk.symbol_rate=Symbolrate * 1000UL;
  Frontend.u.qpsk.fec_inner=FEC_AUTO;
  CHECK(ioctl(fd_frontend,FE_SET_FRONTEND,&Frontend));
  fe_status_t  status = fe_status_t(0);
  usleep(15);
  CHECK(ioctl(fd_frontend, FE_READ_STATUS, &status));
  if (status & FE_HAS_SYNC)
  return true;
        else return false;
}



// --- cPluginRotor ---------------------------------------------------------

class cPluginRotor : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  cStatusMonitor *statusMonitor;
public:
  cPluginRotor(void);
  virtual ~cPluginRotor();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Initialize(void);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { return tr(MAINMENUENTRY); }
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
};

cPluginRotor::cPluginRotor(void)
{
  statusMonitor = NULL;
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginRotor::~cPluginRotor()
{
  // Clean up after yourself!
  delete statusMonitor;
}

const char *cPluginRotor::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginRotor::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginRotor::Initialize(void)
{
  // Initialize any background activities the plugin shall perform.
  return true;
}

bool cPluginRotor::Start(void)   			// TODO: Das geht wahrscheinlich auch einfacher!
{
  // Start any background activities the plugin shall perform.
  statusMonitor = new cStatusMonitor;
  RegisterI18n(Phrases);
  const cSource *source;
  source=Sources.First();
  struct dvb_diseqc_master_cmd cmd;
  sprintf(SourceD[0].name,tr("Not in DISEQC.CONF"));
  int ueberpr;
  cDiseqc *diseqc;
  uchar *codes;
  while (true) {
        diseqc = Diseqcs.Get(source->Code(),12000,'h');
        ueberpr=true;
        if (diseqc) {
         char *CurrentAction = NULL;

           while (ueberpr && (diseqc->Execute(&CurrentAction))!= cDiseqc::daNone)  {
           int n = 0;
           codes = diseqc->Codes(n);
           if (codes) {
	       memcpy(cmd.msg, codes, min(n, int(sizeof(cmd.msg))));
               if (((cmd.msg[0]==0xe0) || (cmd.msg[0]==0xe1)) && ((cmd.msg[1]==0x30) || (cmd.msg[1]==0x31)) && (cmd.msg[2]==0x6b) && ueberpr && (n==4)) {
                  anz++;
                  snprintf(SourceD[anz].name, sizeof(SourceD[anz].name),"%s - %s",cSource::ToString(source->Code()),source->Description());
                  SourceD[anz].Pos=cmd.msg[3];
                  SourceD[anz].code=source->Code();
	          ueberpr=false;
		 } 
             }   
	    } 
        }
	if (ueberpr && ((source->Code() & 0xC000) == 0x8000)) {
           anz++;
           snprintf(SourceD[anz].name, sizeof(SourceD[anz].name),"%s - %s",cSource::ToString(source->Code()),source->Description());
           SourceD[anz].Pos=0;
           SourceD[anz].code=source->Code();
           } 
       if (source->Next())
          source = (cSource *)source->Next();
          else break;
      }
  return true;
}

void cPluginRotor::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdObject *cPluginRotor::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  return new cMainMenuRotor;
}

cMenuSetupPage *cPluginRotor::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuSetupRotor;
}

bool cPluginRotor::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  if      (!strcasecmp(Name, "DVB-Karte"))      DvbKarte = atoi(Value);
  else if (!strcasecmp(Name, "AdressByte"))     AByte = atoi(Value);
  else if (!strcasecmp(Name, "Wiederholen"))    wiederholen = atoi(Value);
  else if (!strcasecmp(Name, "Latitude"))       Lat = atoi(Value);
  else if (!strcasecmp(Name, "SouthNorth"))     SN = atoi(Value);
  else if (!strcasecmp(Name, "Longitude"))      Long = atoi(Value);
  else if (!strcasecmp(Name, "EastWest"))       EW = atoi(Value);
  else if (!strcasecmp(Name, "SLOF"))           SLOF = atoi(Value);
  else if (!strcasecmp(Name, "FrequLo"))        FrequLo = atoi(Value);
  else if (!strcasecmp(Name, "FrequHi"))        FrequHi = atoi(Value);
  else if (!strcasecmp(Name, "UseGotoX"))       UseGotoX = atoi(Value);
  else if (!strcasecmp(Name, "ShowInfo"))       ShowInfo = atoi(Value);
  else
    return false;
  return true;
}

VDRPLUGINCREATOR(cPluginRotor); // Don't touch this!

